@@ -1,15 +1,14 @@
 # To do:
 # Hammering prevention
-#  - Peer finding
 #  - Request trading
 # Negative cache
 # Max-age cache
 # Remove dateutil/req from pagegetter
-# Remove pytz/req from pagegetter
+# Make aws_sdb_reservation_domain optional
 
 from twisted.python.failure import Failure
 
-from uuid import uuid4
+from uuid import uuid4, uuid5, NAMESPACE_DNS
 from twisted.internet import task
 from twisted.internet import reactor
 from twisted.internet.defer import Deferred, DeferredList
@@ -18,7 +17,7 @@ from pagegetter import RequestQueuer, PageGetter
 from unicodeconverter import convertToUTF8
 
 from aws import AmazonS3, AmazonSDB
-
+from twisted.web.client import _parse
 from twisted.web import server
 from twisted.web.resource import Resource
 import time
@@ -29,6 +28,7 @@ from twisted.internet import reactor
 
 from twisted.web import static
 
+import urllib
 import re
 import os
 
@@ -45,7 +45,6 @@ from twisted.internet import threads
 import cPickle
 import sys
 
-
 import pprint
 PrettyPrinter = pprint.PrettyPrinter(indent=4)
 
@@ -57,20 +56,7 @@ from networkaddress import getNetworkAddress
 
 logger = logging.getLogger("main")
 
-def evaluateBoolean( b ):
-    if b == "false" or b == "False":
-        return False
-    elif b == "true" or b == "True":
-        return True
-    elif b == "no" or b == "No":
-        return False
-    elif b == "yes" or b == "Yes":
-        return True
-    else:
-        try:
-            return bool( int(b) )
-        except:
-            return False
+from .evaluateboolean import evaluateBoolean
 
 class AWSpider:
 
@@ -102,14 +88,19 @@ class AWSpider:
                 aws_s3_storage_bucket=None,
                 aws_sdb_coordination_domain=None, 
                 port=8080, 
-                max_requests=50, 
+                max_simultaneous_requests=50,
+                max_requests_per_domain_per_second=1,
+                max_simultaneous_requests_per_domain=5,
                 log_directory=None, 
                 log_level="debug",
                 web_admin=True,
                 web_admin_port=None,
                 time_offset=None,
                 peer_check_interval=5,
-                reservation_check_interval=5):
+                reservation_check_interval=5,
+                hammer_prevention=True ):
+        
+        self.hammer_prevention = hammer_prevention
         
         self.time_offset = time_offset
         self.peer_check_interval = int(peer_check_interval)
@@ -118,7 +109,11 @@ class AWSpider:
         self.port = int(port)
         self.network_information["port"] = self.port
         
-        self.rq = RequestQueuer( max_requests=int(max_requests) )
+        self.rq = RequestQueuer( 
+            max_simultaneous_requests=int(max_simultaneous_requests), 
+            max_requests_per_domain_per_second=int(max_requests_per_domain_per_second), 
+            max_simultaneous_requests_per_domain=int(max_simultaneous_requests_per_domain)
+        )
         
         self.aws_access_key_id = aws_access_key_id
         self.aws_secret_access_key = aws_secret_access_key
@@ -199,8 +194,48 @@ class AWSpider:
         return data
 
     def getPage( self, *args, **kwargs ):
+        if not self.hammer_prevention or len( self.peer_uuids ) == 0:
+            return self.pg.getPage( *args, **kwargs )
+        else:
+            scheme, host, port, path = _parse( args[0] )
+            peer_uuid = self.peer_uuids[ int( uuid5(NAMESPACE_DNS, host).int % len( self.peer_uuids ) ) ]
+            if peer_uuid == self.uuid or self.peers[ peer_uuid ]["active"] == False:
+                return self.pg.getPage( *args, **kwargs )
+            else:
+                
+                parameters = {}
+                parameters["url"] = args[0]
+                if "method" in kwargs:
+                    parameters["method"] = kwargs["method"]   
+                if "postdata" in kwargs: 
+                    parameters["postdata"] = urllib.urlencode( kwargs["postdata"] )
+                if "headers" in kwargs: 
+                    parameters["headers"] = urllib.urlencode( kwargs["headers"] )
+                if "cookies" in kwargs: 
+                    parameters["cookies"] = urllib.urlencode( kwargs["cookies"] )         
+                if "agent" in kwargs:
+                    parameters["agent"] = kwargs["agent"]
+                if "timeout" in kwargs:
+                    parameters["timeout"] = kwargs["timeout"]
+                if "followRedirect" in kwargs:
+                    parameters["followRedirect"] = kwargs["followRedirect"]
+                if "hash_url" in kwargs: 
+                    parameters["hash_url"] = kwargs["hash_url"]
+                if "cache" in kwargs: 
+                    parameters["cache"] = kwargs["cache"]
+                if "prioritize" in kwargs: 
+                    parameters["prioritize"] = kwargs["prioritize"]
+                
+                url = "%s/peer/getpage?%s" % ( self.peers[ peer_uuid ]["uri"], urllib.urlencode(parameters) )
+                logger.debug( "Re-routing request for %s to %s" % (args[0], url) )
+                d = self.rq.getPage(url)
+                d.addErrback( self._getPageErrback, args, kwargs ) 
+                return d
+                
+    def _getPageErrback( self, error, args, kwargs ):
+        logger.error( args[0] + ":" + str(error) )
         return self.pg.getPage( *args, **kwargs )
-    
+
     def getExposedFunctionDetails( self ):
         functions = []
         for key in self.exposed_functions:
@@ -573,10 +608,12 @@ class AWSpider:
         
     def _deleteReservationCallback( self, data, function_name, uuid ):
         logger.info( "Reservation %s, %s successfully deleted." % (function_name, uuid) )
+        return True
         
     def _deleteReservationErrback( self, error, function_name, uuid ):
         logger.error( "Error deleting reservation %s, %s.\n%s" ) % (function_name, uuid, error)
-
+        return False
+        
     def query( self ):
         
         if len(self.functions) > 0:
@@ -746,7 +783,7 @@ class AWSpider:
         for uuid in self.peers:
             if uuid != self.uuid:
                 logger.debug( "Signaling %s to check peers." % self.peers[uuid]["uri"] )
-                d = self.getPage( self.peers[uuid]["uri"] + "/peer/check", cache= -1 )
+                d = self.pg.getPage( self.peers[uuid]["uri"] + "/peer/check", cache=-1 )
                 d.addCallback( self._peerCheckRequestCallback, self.peers[uuid]["uri"] )
                 deferreds.append( d )
 
@@ -856,14 +893,14 @@ class AWSpider:
         if "local_ip" in peer:
             local_ip = peer["local_ip"][0]
             local_url = "http://%s:%s/data/server" % (local_ip, port)
-            d = self.getPage( local_url, timeout=5, cache=-1 )
+            d = self.pg.getPage( local_url, timeout=5, cache=-1 )
             d.addCallback( self._verifyPeerLocalIPCallback, uuid, local_ip, port )
             deferreds.append( d )
             
         if "public_ip" in peer:
             public_ip = peer["public_ip"][0]
             public_url = "http://%s:%s/data/server" % (public_ip, port)
-            d = self.getPage( public_url, timeout=5, cache=-1  )         
+            d = self.pg.getPage( public_url, timeout=5, cache=-1  )         
             d.addCallback( self._verifyPeerPublicIPCallback, uuid, public_ip, port )
             deferreds.append( d )
         
