@@ -7,60 +7,116 @@ import sha
 import time
 import dateutil.parser
 import datetime
-import pytz
 import urllib
 
 import logging
 logger = logging.getLogger("main")
 
+import sys
+from twisted.python.failure import Failure
+
+from datetime import tzinfo, timedelta
+
+from twisted.internet import task
+
+# A UTC class.
+
+class UTC(tzinfo):
+    ZERO = timedelta(0)
+    def utcoffset(self, dt):
+        return self.ZERO
+    def tzname(self, dt):
+        return "UTC"
+    def dst(self, dt):
+        return self.ZERO
+        
+utc = UTC()
+
+def pending_len( d ):
+    if d:
+        return sum( map( len, d.values() ) )
+    else:
+        return 0
+        
 class RequestQueuer():
 
-    def __init__(self, max_requests=50):
+    def __init__(self, max_simultaneous_requests=50, max_requests_per_domain_per_second=1, max_simultaneous_requests_per_domain=5 ):
     
-        self.max_requests = max_requests
-            
-        self.pending = []
-        self.active = 0
-
-    def checkActive(self):
+        self.max_simultaneous_requests = max_simultaneous_requests
+        self.min_request_interval_per_domain = 1.0 / float(max_requests_per_domain_per_second)
+        self.max_simultaneous_requests_per_domain = max_simultaneous_requests_per_domain
         
-        while self.active < self.max_requests:
+        self.pending_requests = {}
+        self.pending_last_request = {}
+        self.active_requests = {}
+        self.min_request_interval_per_domains = {}
+        self.max_simultaneous_requests_per_domains = {}
+        
+        task.LoopingCall( self.removeEmptyPendingHostLists ).start( 30 )
+    
+    def setHostMaxRequestsPerSecond( self, host, max_requests_per_second ):
+        if max_requests_per_second == 0:
+            self.min_request_interval_per_domains[ host ] = 0
+        else:        
+            min_request_interval = 1.0 / float(max_requests_per_second)
+            self.min_request_interval_per_domains[ host ] = min_request_interval
+    
+    def setHostMaxSimultaneousRequests( self, host, max_simultaneous_requests ):
+        if max_simultaneous_requests == 0:
+            self.max_simultaneous_requests_per_domains[ host ] = self.max_simultaneous_requests
+        else:
+            self.max_simultaneous_requests_per_domains[ host ] = max_simultaneous_requests
             
-            if len( self.pending ) > 0:
+    def checkActive(self):
                 
-                req = self.pending.pop(0)
-                
-                self.active = self.active + 1
-
-                d = self._getPage(  
-                    req["url"], 
-                    method=req["method"], 
-                    postdata=req["postdata"],
-                    headers=req["headers"], 
-                    agent=req["agent"], 
-                    timeout=req["timeout"], 
-                    cookies=req["cookies"],
-                    followRedirect=req["followRedirect"]
-                )
-
-                d.addCallback( self.requestComplete, req["deferred"] ).addErrback( self.requestError, req["deferred"] )
-
-            else:
-                break
-
-    def requestComplete( self, response, deferred ):
-        self.active = self.active - 1
+        while pending_len( self.pending_requests ) < self.max_simultaneous_requests and pending_len( self.pending_requests ) > 0:
+            
+            in_loop_request_count = 0
+            
+            for host in self.pending_requests:
+                if len( self.pending_requests[ host ] ) > 0 and \
+                time.time() - self.pending_last_request.get( host, 0 ) > self.min_request_interval_per_domains.get( host, self.min_request_interval_per_domain ) and \
+                self.active_requests.get( host, 0 ) < self.max_simultaneous_requests_per_domains.get( host, self.max_simultaneous_requests_per_domain ):
+                    
+                    in_loop_request_count += 1
+                    req = self.pending_requests[ host ].pop(0)
+                    d = self._getPage(  
+                        req["url"], 
+                        method=req["method"], 
+                        postdata=req["postdata"],
+                        headers=req["headers"], 
+                        agent=req["agent"], 
+                        timeout=req["timeout"], 
+                        cookies=req["cookies"],
+                        followRedirect=req["followRedirect"]
+                    )
+                    d.addCallback( self.requestComplete, req["deferred"], host )
+                    d.addErrback( self.requestError, req["deferred"], host )
+                    self.pending_last_request[ host ] = time.time()
+                    self.active_requests[ host ] = self.active_requests.get( host, 0 ) + 1
+            
+            if in_loop_request_count == 0:
+                reactor.callLater( .1, self.checkActive)
+                return
+        
+    def removeEmptyPendingHostLists( self ):
+        hosts_to_be_removed = filter( lambda host:len(self.pending_requests[host]) == 0, self.pending_requests.keys() )
+        for host in hosts_to_be_removed:
+            del self.pending_requests[host]
+        
+    def requestComplete( self, response, deferred, host ):
+        self.active_requests[ host ] -= 1
         self.checkActive()
         deferred.callback( response )
         return None
-    
-    def requestError( self, error, deferred ):
-        self.active = self.active - 1
+
+    def requestError( self, error, deferred, host ):
+        self.active_requests[ host ] -= 1
         self.checkActive()
         deferred.errback( error )  
         return None
     
-    def getPage(self, url, last_modified=None, etag=None, method='GET', postdata=None, headers=None, agent="Offbeat Guides", timeout=60, cookies=None, followRedirect=1, prioritize=False ):
+    def getPage(self, url, last_modified=None, etag=None, method='GET', postdata=None, headers=None, agent="AWSpider", timeout=60, cookies=None, followRedirect=1, prioritize=False ):
 
         if headers is None:
             headers = {}
@@ -87,11 +143,17 @@ class RequestQueuer():
             "deferred":deferred 
         }
                 
+        host = _parse(url)[1]
+        
+        if host not in self.pending_requests:
+            self.pending_requests[ host ] = []
+        
         if prioritize:
-            self.pending.insert( 0, req )
+            self.pending_requests[ host ].insert( 0, req )
         else:
-            self.pending.append( req )
-            
+            self.pending_requests[ host ].append( req )
+        
+
         self.checkActive()
 
         return deferred
@@ -115,17 +177,12 @@ class RequestQueuer():
         )
 
         if scheme == 'https':
-            
             try:
                 from twisted.internet import ssl
-            except Exception, e:
-                logger.error( str(e) )
-            
-            try:
                 contextFactory = ssl.ClientContextFactory()
                 reactor.connectSSL(host, port, factory, contextFactory, timeout=timeout)
             except Exception, e:
-                logger.error( str(e) )
+                return Failure(exc_value=sys.exc_value, exc_type=sys.exc_type, exc_tb=sys.exc_traceback)
 
         else:
             reactor.connectTCP(host, port, factory, timeout=timeout)
@@ -144,6 +201,8 @@ class RequestQueuer():
         
         return error
 
+
+        
 class PageGetter:
     
     def __init__( self, s3, aws_s3_bucket, rq=None ):
@@ -156,7 +215,7 @@ class PageGetter:
         else:
             self.rq = rq
     
-    def getPage(self, url, method='GET', postdata=None, headers=None, agent="AWSpider", timeout=60, cookies=None, followRedirect=1, hash_url=None, cache=0, metadata=None, prioritize=True ):
+    def getPage(self, url, method='GET', postdata=None, headers=None, agent="AWSpider", timeout=60, cookies=None, followRedirect=1, hash_url=None, cache=0, prioritize=True ):
 
         cache=int(cache)
         
@@ -166,9 +225,6 @@ class PageGetter:
         url = convertToUTF8( url )
         if hash_url is not None:
             hash_url = convertToUTF8( hash_url )
-
-        if metadata is None:
-            metadata = {}
         
         if isinstance( postdata, dict ):
             for key in postdata:
@@ -188,10 +244,8 @@ class PageGetter:
         parameters_to_be_hashed = cPickle.dumps( [headers, agent, cookies] )
         
         if hash_url is None:
-            metadata["url"] = url
             request_hash = sha.new( url + parameters_to_be_hashed ).hexdigest()
         else:
-            metadata["url"] = hash_url
             request_hash = sha.new( hash_url + parameters_to_be_hashed ).hexdigest()
         
         if cache == -1:
@@ -221,7 +275,7 @@ class PageGetter:
         try:
             if "cache-expires" in data["headers"]:
                 expires = dateutil.parser.parse(data["headers"]["cache-expires"][0])
-                now = datetime.datetime.now( pytz.timezone("UTC") )
+                now = datetime.datetime.now( utc )
                 if expires < now:
                     logger.debug( "Stored data request %s for URL %s is not stale. Getting from S3." % (request_hash, url) )
                     d = self.s3.getObject( self.aws_s3_bucket, request_hash )
